"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var tslint_1 = require("tslint");
var component_data_1 = require("../material/component-data");
var component_walker_1 = require("../tslint/component-walker");
var literal_1 = require("../typescript/literal");
/**
 * Rule that walks through every component decorator and updates their inline or external
 * templates.
 */
var Rule = (function (_super) {
    __extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.apply = function (sourceFile) {
        return this.applyWithWalker(new SwitchTemplateCaaNamesWalker(sourceFile, this.getOptions()));
    };
    return Rule;
}(tslint_1.Rules.AbstractRule));
exports.Rule = Rule;
var SwitchTemplateCaaNamesWalker = (function (_super) {
    __extends(SwitchTemplateCaaNamesWalker, _super);
    function SwitchTemplateCaaNamesWalker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SwitchTemplateCaaNamesWalker.prototype.visitInlineTemplate = function (template) {
        var _this = this;
        this.replaceNamesInTemplate(template, template.getText()).forEach(function (replacement) {
            var fix = replacement.replacement;
            var ruleFailure = new tslint_1.RuleFailure(template.getSourceFile(), fix.start, fix.end, replacement.message, _this.getRuleName(), fix);
            _this.addFailure(ruleFailure);
        });
    };
    SwitchTemplateCaaNamesWalker.prototype.visitExternalTemplate = function (template) {
        var _this = this;
        this.replaceNamesInTemplate(template, template.getFullText()).forEach(function (replacement) {
            var fix = replacement.replacement;
            var ruleFailure = new tslint_1.RuleFailure(template, fix.start + 1, fix.end + 1, replacement.message, _this.getRuleName(), fix);
            _this.addFailure(ruleFailure);
        });
    };
    /**
     * Replaces the outdated name in the template with the new one and returns an updated template.
     */
    SwitchTemplateCaaNamesWalker.prototype.replaceNamesInTemplate = function (node, templateContent) {
        var _this = this;
        var replacements = [];
        component_data_1.cssNames.forEach(function (name) {
            if (!name.whitelist || name.whitelist.html) {
                _this.createReplacementsForOffsets(node, name, literal_1.findAll(templateContent, name.replace))
                    .forEach(function (replacement) {
                    replacements.push({
                        message: "Found deprecated CSS class \"" + chalk_1.red(name.replace) + "\" which has been" +
                            (" renamed to \"" + chalk_1.green(name.replaceWith) + "\""),
                        replacement: replacement
                    });
                });
            }
        });
        return replacements;
    };
    SwitchTemplateCaaNamesWalker.prototype.createReplacementsForOffsets = function (node, update, offsets) {
        var _this = this;
        return offsets.map(function (offset) { return _this.createReplacement(node.getStart() + offset, update.replace.length, update.replaceWith); });
    };
    return SwitchTemplateCaaNamesWalker;
}(component_walker_1.ComponentWalker));
exports.SwitchTemplateCaaNamesWalker = SwitchTemplateCaaNamesWalker;
//# sourceMappingURL=switchTemplateCssNamesRule.js.map